
#include <stdlib.h>
#include <string.h>
#ifdef SDL
#include "sdl.h"
#else
#include <pspkernel.h>
#include <pspaudio.h>
#include <pspaudiolib.h>
#endif
#include "audio.h"
#include "system.h"
#include "def.h"

struct _AUDIO_BUFFER Audio_Buffer;

#ifdef SDL

BOOL sdl_audio_init = FALSE;
BOOL sdl_audio_open = FALSE;

void Audio_Fill(void *udata, unsigned char *stream, int size)
{
  if(Audio_Buffer.Filled < size) memset(stream, 0, size);
  else
  {
    memcpy(stream, Audio_Buffer.Data, size);
    Audio_Buffer.Filled -= size;
    //do { Audio_Buffer.Filled -= size; } while(Audio_Buffer.Filled > 2 * size);
    memmove(Audio_Buffer.Data, ((char *)Audio_Buffer.Data) + size, Audio_Buffer.Filled);
    Audio_Buffer.CurrentPosition = (short *)(((char *)Audio_Buffer.Data) + Audio_Buffer.Filled);
  }
}

#else

BOOL psp_audio_init = FALSE;
BOOL audio_locked, audio_paused;

void Audio_Fill(void *buf, unsigned int len, void *data)
{
  len *= 4;
  if(audio_paused) memset(buf, 0, len);
  else
  {
    while(audio_locked) sceKernelDelayThreadCB(500);
    if((unsigned)Audio_Buffer.Filled < len) memset(buf, 0, len);
    else
    {
      memcpy(buf, Audio_Buffer.Data, len);
      Audio_Buffer.Filled -= len;
      //do { Audio_Buffer.Filled -= size; } while(Audio_Buffer.Filled > 2 * size);
      memmove(Audio_Buffer.Data, ((char *)Audio_Buffer.Data) + len, Audio_Buffer.Filled);
      Audio_Buffer.CurrentPosition = (short *)(((char *)Audio_Buffer.Data) + Audio_Buffer.Filled);
    }
  }
}

#endif

void Audio_Init(void)
{
#ifdef SDL

  if(!sdl_audio_init)
  {
    if(SDL_InitSubSystem(SDL_INIT_AUDIO) < 0) exit(0);
    sdl_audio_init = TRUE;
  }

  if(!sdl_audio_open)
  {
    SDL_AudioSpec sound;
  
    memset(&sound, 0, sizeof(SDL_AudioSpec));
    sound.freq     = 44100;
    sound.format   = AUDIO_S16;
    sound.channels = 2;
    sound.samples  = 1024;
    sound.size     = sound.samples * 4;
    sound.callback = Audio_Fill;
    if((SDL_OpenAudio(&sound, (SDL_AudioSpec *)0) < 0)) { Audio_Quit(); exit(0); }
    SDL_PauseAudio(1);
    sdl_audio_open = TRUE;
  }

#else

  if(!psp_audio_init)
  {
    audio_locked = FALSE;
    pspAudioInit();
    pspAudioSetChannelCallback(1, Audio_Fill, (void *)0);
    psp_audio_init = audio_paused = TRUE;
  }

#endif

  memset(&Audio_Buffer, 0, sizeof(Audio_Buffer));
  Audio_Buffer.CurrentPosition = Audio_Buffer.Data;
}

void Audio_Quit(void)
{
#ifdef SDL

  if(sdl_audio_open) { SDL_CloseAudio(); sdl_audio_open = FALSE; }
  if(sdl_audio_init) { SDL_QuitSubSystem(SDL_INIT_AUDIO); sdl_audio_init = FALSE; }

#else

  if(psp_audio_init) { pspAudioEnd(); psp_audio_init = FALSE; }

#endif
}
