/** Z80: portable Z80 emulator *******************************/
/**                                                         **/
/**                         CodesED.h                       **/
/**                                                         **/
/** This file contains implementation for the ED table of   **/
/** Z80 commands. It is included from Z80.c.                **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin 1994-2002                 **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/

/** This is a special patch for emulating BIOS calls: ********/
case DB_FE:     PatchZ80();break;
/*************************************************************/

case ADC_HL_BC: M_ADCW(BC);break;
case ADC_HL_DE: M_ADCW(DE);break;
case ADC_HL_HL: M_ADCW(HL);break;
case ADC_HL_SP: M_ADCW(SP);break;

case SBC_HL_BC: M_SBCW(BC);break;
case SBC_HL_DE: M_SBCW(DE);break;
case SBC_HL_HL: M_SBCW(HL);break;
case SBC_HL_SP: M_SBCW(SP);break;

case LD_xWORDe_HL:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  WrZ80(J.W++,Z80.HL.B.l);
  WrZ80(J.W,Z80.HL.B.h);
  break;
case LD_xWORDe_DE:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  WrZ80(J.W++,Z80.DE.B.l);
  WrZ80(J.W,Z80.DE.B.h);
  break;
case LD_xWORDe_BC:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  WrZ80(J.W++,Z80.BC.B.l);
  WrZ80(J.W,Z80.BC.B.h);
  break;
case LD_xWORDe_SP:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  WrZ80(J.W++,Z80.SP.B.l);
  WrZ80(J.W,Z80.SP.B.h);
  break;

case LD_HL_xWORDe:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  Z80.HL.B.l=RdZ80(J.W++);
  Z80.HL.B.h=RdZ80(J.W);
  break;
case LD_DE_xWORDe:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  Z80.DE.B.l=RdZ80(J.W++);
  Z80.DE.B.h=RdZ80(J.W);
  break;
case LD_BC_xWORDe:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  Z80.BC.B.l=RdZ80(J.W++);
  Z80.BC.B.h=RdZ80(J.W);
  break;
case LD_SP_xWORDe:
  J.B.l=RdZ80(Z80.PC.W++);
  J.B.h=RdZ80(Z80.PC.W++);
  Z80.SP.B.l=RdZ80(J.W++);
  Z80.SP.B.h=RdZ80(J.W);
  break;

case RRD:
  I=RdZ80(Z80.HL.W);
  J.B.l=(I>>4)|(Z80.AF.B.h<<4);
  WrZ80(Z80.HL.W,J.B.l);
  Z80.AF.B.h=(I&0x0F)|(Z80.AF.B.h&0xF0);
  Z80.AF.B.l=PZSTable[Z80.AF.B.h]|(Z80.AF.B.l&C_FLAG);
  break;
case RLD:
  I=RdZ80(Z80.HL.W);
  J.B.l=(I<<4)|(Z80.AF.B.h&0x0F);
  WrZ80(Z80.HL.W,J.B.l);
  Z80.AF.B.h=(I>>4)|(Z80.AF.B.h&0xF0);
  Z80.AF.B.l=PZSTable[Z80.AF.B.h]|(Z80.AF.B.l&C_FLAG);
  break;

case LD_A_I:
  Z80.AF.B.h=Z80.I;
  Z80.AF.B.l=(Z80.AF.B.l&C_FLAG)|(Z80.IFF&IFF_2? P_FLAG:0)|ZSTable[Z80.AF.B.h];
  break;

case LD_A_R:
  Z80.R++;
  Z80.AF.B.h=(byte)(Z80.R-Z80.ICount);
  Z80.AF.B.l=(Z80.AF.B.l&C_FLAG)|(Z80.IFF&IFF_2? P_FLAG:0)|ZSTable[Z80.AF.B.h];
  break;

case LD_I_A:   Z80.I=Z80.AF.B.h;break;
case LD_R_A:   break;

case IM_0:     Z80.IFF&=~(IFF_IM1|IFF_IM2);break;
case IM_1:     Z80.IFF=(Z80.IFF&~IFF_IM2)|IFF_IM1;break;
case IM_2:     Z80.IFF=(Z80.IFF&~IFF_IM1)|IFF_IM2;break;

case RETI:     M_RET;break;
case RETN:     if(Z80.IFF&IFF_2) Z80.IFF|=IFF_1; else Z80.IFF&=~IFF_1;
               M_RET;break;

case NEG:      I=Z80.AF.B.h;Z80.AF.B.h=0;M_SUB(I);break;

case IN_B_xC:  M_IN(Z80.BC.B.h);break;
case IN_C_xC:  M_IN(Z80.BC.B.l);break;
case IN_D_xC:  M_IN(Z80.DE.B.h);break;
case IN_E_xC:  M_IN(Z80.DE.B.l);break;
case IN_H_xC:  M_IN(Z80.HL.B.h);break;
case IN_L_xC:  M_IN(Z80.HL.B.l);break;
case IN_A_xC:  M_IN(Z80.AF.B.h);break;
case IN_F_xC:  M_IN(J.B.l);break;

case OUT_xC_B: OutZ80(Z80.BC.B.l,Z80.BC.B.h);break;
case OUT_xC_C: OutZ80(Z80.BC.B.l,Z80.BC.B.l);break;
case OUT_xC_D: OutZ80(Z80.BC.B.l,Z80.DE.B.h);break;
case OUT_xC_E: OutZ80(Z80.BC.B.l,Z80.DE.B.l);break;
case OUT_xC_H: OutZ80(Z80.BC.B.l,Z80.HL.B.h);break;
case OUT_xC_L: OutZ80(Z80.BC.B.l,Z80.HL.B.l);break;
case OUT_xC_A: OutZ80(Z80.BC.B.l,Z80.AF.B.h);break;

case INI:
  WrZ80(Z80.HL.W++,InZ80(Z80.BC.B.l));
  Z80.BC.B.h--;
  Z80.AF.B.l=N_FLAG|(Z80.BC.B.h? 0:Z_FLAG);
  break;

case INIR:
  do
  {
    WrZ80(Z80.HL.W++,InZ80(Z80.BC.B.l));
    Z80.BC.B.h--;Z80.ICount-=21;
  }
  while(Z80.BC.B.h&&(Z80.ICount>0));
  if(Z80.BC.B.h) { Z80.AF.B.l=N_FLAG;Z80.PC.W-=2; }
  else { Z80.AF.B.l=Z_FLAG|N_FLAG;Z80.ICount+=5; }
  break;

case IND:
  WrZ80(Z80.HL.W--,InZ80(Z80.BC.B.l));
  Z80.BC.B.h--;
  Z80.AF.B.l=N_FLAG|(Z80.BC.B.h? 0:Z_FLAG);
  break;

case INDR:
  do
  {
    WrZ80(Z80.HL.W--,InZ80(Z80.BC.B.l));
    Z80.BC.B.h--;Z80.ICount-=21;
  }
  while(Z80.BC.B.h&&(Z80.ICount>0));
  if(Z80.BC.B.h) { Z80.AF.B.l=N_FLAG;Z80.PC.W-=2; }
  else { Z80.AF.B.l=Z_FLAG|N_FLAG;Z80.ICount+=5; }
  break;

case OUTI:
  I=RdZ80(Z80.HL.W++);
  OutZ80(Z80.BC.B.l,I);
  Z80.BC.B.h--;
  Z80.AF.B.l=N_FLAG|(Z80.BC.B.h? 0:Z_FLAG)|(Z80.HL.B.l+I>255? (C_FLAG|H_FLAG):0);
  break;

case OTIR:
  do
  {
    I=RdZ80(Z80.HL.W++);
    OutZ80(Z80.BC.B.l,I);
    Z80.BC.B.h--;
    Z80.ICount-=21;
  }
  while(Z80.BC.B.h&&(Z80.ICount>0));
  if(Z80.BC.B.h)
  {
    Z80.AF.B.l=N_FLAG|(Z80.HL.B.l+I>255? (C_FLAG|H_FLAG):0);
    Z80.PC.W-=2;
  }
  else
  {
    Z80.AF.B.l=Z_FLAG|N_FLAG|(Z80.HL.B.l+I>255? (C_FLAG|H_FLAG):0);
    Z80.ICount+=5;
  }
  break;

case OUTD:
  I=RdZ80(Z80.HL.W--);
  OutZ80(Z80.BC.B.l,I);
  Z80.BC.B.h--;
  Z80.AF.B.l=N_FLAG|(Z80.BC.B.h? 0:Z_FLAG)|(Z80.HL.B.l+I>255? (C_FLAG|H_FLAG):0);
  break;

case OTDR:
  do
  {
    I=RdZ80(Z80.HL.W--);
    OutZ80(Z80.BC.B.l,I);
    Z80.BC.B.h--;
    Z80.ICount-=21;
  }
  while(Z80.BC.B.h&&(Z80.ICount>0));
  if(Z80.BC.B.h)
  {
    Z80.AF.B.l=N_FLAG|(Z80.HL.B.l+I>255? (C_FLAG|H_FLAG):0);
    Z80.PC.W-=2;
  }
  else
  {
    Z80.AF.B.l=Z_FLAG|N_FLAG|(Z80.HL.B.l+I>255? (C_FLAG|H_FLAG):0);
    Z80.ICount+=5;
  }
  break;

case LDI:
  WrZ80(Z80.DE.W++,RdZ80(Z80.HL.W++));
  Z80.BC.W--;
  Z80.AF.B.l=(Z80.AF.B.l&~(N_FLAG|H_FLAG|P_FLAG))|(Z80.BC.W? P_FLAG:0);
  break;

case LDIR:
  do
  {
    WrZ80(Z80.DE.W++,RdZ80(Z80.HL.W++));
    Z80.BC.W--;Z80.ICount-=21;
  }
  while(Z80.BC.W&&(Z80.ICount>0));
  Z80.AF.B.l&=~(N_FLAG|H_FLAG|P_FLAG);
  if(Z80.BC.W) { Z80.AF.B.l|=N_FLAG;Z80.PC.W-=2; }
  else Z80.ICount+=5;
  break;

case LDD:
  WrZ80(Z80.DE.W--,RdZ80(Z80.HL.W--));
  Z80.BC.W--;
  Z80.AF.B.l=(Z80.AF.B.l&~(N_FLAG|H_FLAG|P_FLAG))|(Z80.BC.W? P_FLAG:0);
  break;

case LDDR:
  do
  {
    WrZ80(Z80.DE.W--,RdZ80(Z80.HL.W--));
    Z80.BC.W--;Z80.ICount-=21;
  }
  while(Z80.BC.W&&(Z80.ICount>0));
  Z80.AF.B.l&=~(N_FLAG|H_FLAG|P_FLAG);
  if(Z80.BC.W) { Z80.AF.B.l|=N_FLAG;Z80.PC.W-=2; }
  else Z80.ICount+=5;
  break;

case CPI:
  I=RdZ80(Z80.HL.W++);
  J.B.l=Z80.AF.B.h-I;
  Z80.BC.W--;
  Z80.AF.B.l =
    N_FLAG|(Z80.AF.B.l&C_FLAG)|ZSTable[J.B.l]|
    ((Z80.AF.B.h^I^J.B.l)&H_FLAG)|(Z80.BC.W? P_FLAG:0);
  break;

case CPIR:
  do
  {
    I=RdZ80(Z80.HL.W++);
    J.B.l=Z80.AF.B.h-I;
    Z80.BC.W--;Z80.ICount-=21;
  }  
  while(Z80.BC.W&&J.B.l&&(Z80.ICount>0));
  Z80.AF.B.l =
    N_FLAG|(Z80.AF.B.l&C_FLAG)|ZSTable[J.B.l]|
    ((Z80.AF.B.h^I^J.B.l)&H_FLAG)|(Z80.BC.W? P_FLAG:0);
  if(Z80.BC.W&&J.B.l) Z80.PC.W-=2; else Z80.ICount+=5;
  break;  

case CPD:
  I=RdZ80(Z80.HL.W--);
  J.B.l=Z80.AF.B.h-I;
  Z80.BC.W--;
  Z80.AF.B.l =
    N_FLAG|(Z80.AF.B.l&C_FLAG)|ZSTable[J.B.l]|
    ((Z80.AF.B.h^I^J.B.l)&H_FLAG)|(Z80.BC.W? P_FLAG:0);
  break;

case CPDR:
  do
  {
    I=RdZ80(Z80.HL.W--);
    J.B.l=Z80.AF.B.h-I;
    Z80.BC.W--;Z80.ICount-=21;
  }
  while(Z80.BC.W&&J.B.l);
  Z80.AF.B.l =
    N_FLAG|(Z80.AF.B.l&C_FLAG)|ZSTable[J.B.l]|
    ((Z80.AF.B.h^I^J.B.l)&H_FLAG)|(Z80.BC.W? P_FLAG:0);
  if(Z80.BC.W&&J.B.l) Z80.PC.W-=2; else Z80.ICount+=5;
  break;
