  
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspctrl.h>
#include <psppower.h>
#include "config.h"
#include "video.h"
#include "audio.h"
#include "system.h"
#include "text.h"
#include "menu.h"
#include "language.h"
#include "def.h"

PSP_MODULE_INFO("PSPMaster", 0, 1, 1);

void app_exit(void)
{
  System_Quit();
  Audio_Quit();
  Video_Quit();
  Language_Free();
  Config_Save("config.c");
  sceDisplayWaitVblankStart();
  sceKernelExitGame();
}

// Exit callback
int exit_callback(int arg1, int arg2, void *common)
{
  app_exit();
  return 0;
}

// Callback thread
int callback_thread(SceSize args, void *argp)
{
  int cbid;

  cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
  sceKernelRegisterExitCallback(cbid);

  sceKernelSleepThreadCB();

  return 0;
}

// Sets up the callback thread and returns its thread id
static int setup_callbacks(void)
{
  int thid = 0;

  thid = sceKernelCreateThread("update_thread", callback_thread, 0x11, 0xFA0, 0, 0);
  if(thid >= 0) sceKernelStartThread(thid, 0, 0);

  return thid;
} 

void set_clock_freq(int cpu, int ram, int bus)
{
  if(cpu > 222 && ram > 222 && bus > 111)
    scePowerSetClockFrequency(cpu, ram, bus);
  else
  {
    scePowerSetCpuClockFrequency(cpu);
    scePowerSetBusClockFrequency(bus);
  }
}

#define CHECK_KEY(K) (psp_pad.Buttons & K)

int main(void)
{
  unsigned long now, next_time, wait_time, delay_input;
  SceCtrlData   psp_pad;
  BOOL          state_saved = FALSE, state_loaded = FALSE;
  int           turbo       = 0, vsync = VSync;
  int           frame       = 0;
  const int     fast[2][8]  = { { 0, 0, 0, 0, 0, 0, 0, 0 }, { 1, 1, 1, 1, 1, 1, 1, 0 } };

  setup_callbacks();
  
  // loading configuration file
  Config_Load("config.c");

  set_clock_freq(CPUFreq, RAMFreq, BUSFreq);

  Language_Load(LangFile);
  Text_LoadFont(FontsFile, FontName, FontType);

  Video_Init();
  Audio_Init();
  Video_SetMode(VM_MENU);
  sceCtrlSetSamplingCycle(0);
  sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

  atexit(app_exit);

  {
    char fn[MAX_PATH];
    Video_SetMode(VM_MENU);
    while(!Menu_OpenROM(fn));
    Video_SetMode(VM_EMULATION);
    if(System_Init(fn))
    {
      char str[MAX_STRING], msg[MAX_STRING + MAX_FILE];
      Language_GetString("Message_ROMLoadOK", "ROM \"%s\" loaded", str, MAX_STRING);
      sprintf(msg, str, Cart.Name);
      Video_ShowMessage(msg, DEFAULT_MESSAGE_TIME);
    }
    else exit(0);
  }

  delay_input = sceKernelLibcClock() + (500 * 1000);
  
  Audio_Pause(FALSE);

  next_time = sceKernelLibcClock() + 16667;
  while(1)
  {
_do_frame:
    if(delay_input)
    {
      psp_pad.Buttons = 0;
      psp_pad.Lx = psp_pad.Ly = 128;
      if(sceKernelLibcClock() >= delay_input) delay_input = 0;
    }
    else sceCtrlPeekBufferPositive(&psp_pad, 1);

    if(CHECK_KEY(Keys.Menu))
    {
      unsigned long clk;
      Audio_Pause(TRUE);
      Video_SetMode(VM_MENU);
      Menu_Main();
      Video_SetMode(VM_EMULATION);
      Audio_Pause(FALSE);
      clk         = sceKernelLibcClock();
      next_time   = clk + 16667;
      delay_input = clk + (500 * 1000);
      vsync       = VSync;
      turbo       = 0;
    }

    if(UseAnalogAsDButton)
    {
      Joypad[0].Up     = (CHECK_KEY(Keys.P1_Up)    || (psp_pad.Ly < 64))  ? TRUE : FALSE;
      Joypad[0].Down   = (CHECK_KEY(Keys.P1_Down)  || (psp_pad.Ly > 192)) ? TRUE : FALSE;
      Joypad[0].Left   = (CHECK_KEY(Keys.P1_Left)  || (psp_pad.Lx < 64))  ? TRUE : FALSE;
      Joypad[0].Right  = (CHECK_KEY(Keys.P1_Right) || (psp_pad.Lx > 192)) ? TRUE : FALSE;
    }
    else
    {
      Joypad[0].Up     = CHECK_KEY(Keys.P1_Up)    ? TRUE : FALSE;
      Joypad[0].Down   = CHECK_KEY(Keys.P1_Down)  ? TRUE : FALSE;
      Joypad[0].Left   = CHECK_KEY(Keys.P1_Left)  ? TRUE : FALSE;
      Joypad[0].Right  = CHECK_KEY(Keys.P1_Right) ? TRUE : FALSE;
    }
    Joypad[0].One    = (CHECK_KEY(Keys.P1_One) || CHECK_KEY(Keys.P1_OneTwo)) ? TRUE : FALSE;
    Joypad[0].Two    = (CHECK_KEY(Keys.P1_Two) || CHECK_KEY(Keys.P1_OneTwo)) ? TRUE : FALSE;
    Joypad[0].Start  = CHECK_KEY(Keys.P1_Start) ? TRUE : FALSE;
    Joypad[0].Reset  = CHECK_KEY(Keys.P1_Reset) ? TRUE : FALSE;
    Joypad[1].Up     = CHECK_KEY(Keys.P2_Up)    ? TRUE : FALSE;
    Joypad[1].Down   = CHECK_KEY(Keys.P2_Down)  ? TRUE : FALSE;
    Joypad[1].Left   = CHECK_KEY(Keys.P2_Left)  ? TRUE : FALSE;
    Joypad[1].Right  = CHECK_KEY(Keys.P2_Right) ? TRUE : FALSE;
    Joypad[1].One    = (CHECK_KEY(Keys.P2_One) || CHECK_KEY(Keys.P2_OneTwo)) ? TRUE : FALSE;
    Joypad[1].Two    = (CHECK_KEY(Keys.P2_Two) || CHECK_KEY(Keys.P2_OneTwo)) ? TRUE : FALSE;
    Joypad[1].Start  = CHECK_KEY(Keys.P2_Start) ? TRUE : FALSE;

    if(CHECK_KEY(Keys.P1_Reset))
    {
      if(System.Flags & SF_GAME_GEAR) { System_Reset(FALSE); goto _do_frame; }
      else Joypad[0].Reset = TRUE;
    }
    else Joypad[0].Reset = FALSE;
    Joypad[1].Reset = CHECK_KEY(Keys.P2_Reset) ? TRUE : FALSE;

    if(CHECK_KEY(Keys.LoadState))
    {
      if(!state_loaded)
      {
#if MAX_PATH > MAX_STRING
        char fn [MAX_PATH];
#else
        char fn [MAX_STRING];
#endif
        char msg[MAX_STRING];
        sprintf(fn, "%s%.8lX.st%i", StateFolder, Cart.CRC, StateSlot);
        if(System_LoadState(fn))
          Language_GetString("Message_LoadState", "State %i loaded", msg, MAX_STRING);
        else
          Language_GetString("Message_UnableToLoadState", "Unable to load state %i", msg, MAX_STRING);
        state_loaded = TRUE;
        sprintf(fn, msg, StateSlot);
        Video_ShowMessage(fn, DEFAULT_MESSAGE_TIME);
      }
    }
    else state_loaded = FALSE;

    if(CHECK_KEY(Keys.Turbo))
    {
      if(!turbo)
      {
        Audio_Pause(TRUE);
        turbo = 1;
        vsync = 0;
        set_clock_freq(333, 333, 166);
      }
    }
    else if(turbo)
    {
      Audio_Pause(FALSE);
      turbo = 0;
      vsync = VSync;
      set_clock_freq(CPUFreq, RAMFreq, BUSFreq);
    }

    System_Frame(fast[turbo][frame]);

    if(CHECK_KEY(Keys.SaveState))
    {
      if(!state_saved)
      {
#if MAX_PATH > MAX_STRING
        char fn [MAX_PATH];
#else
        char fn [MAX_STRING];
#endif
        char msg[MAX_STRING];
        // running bios, can't save state
        if(System.Flags & SF_BIOS) 
        {
          Language_GetString("Message_CantSaveStateInBios", "Can't save state while in BIOS", msg, MAX_STRING);
          Video_ShowMessage(msg, DEFAULT_MESSAGE_TIME);
        }
        else
        {
          sprintf(fn, "%s%.8lX.st%i", StateFolder, Cart.CRC, StateSlot);
          if(System_SaveState(fn))
            Language_GetString("Message_SaveState", "State %i saved", msg, MAX_STRING);
          else
            Language_GetString("Message_UnableToSaveState", "Unable to save state %i", msg, MAX_STRING);
          sprintf(fn, msg, StateSlot);
          Video_ShowMessage(fn, DEFAULT_MESSAGE_TIME);
        }
        state_saved = TRUE;
      }
    }
    else state_saved = FALSE;

    if(!fast[turbo][frame])
    {
      Video_BlitEmulation();
      Video_Flip(vsync);
    }

    if(!turbo)
    {
      now = sceKernelLibcClock();
      wait_time = next_time <= now ? 0 : next_time - now;
      if(wait_time > 16667) wait_time = 16667;
      sceKernelDelayThreadCB(wait_time);
      next_time += 16667;
    }

    frame = (frame + 1) & 0x07;
  }

  sceKernelSleepThread();
  app_exit();

  return 0;
}
