
.globl BGNameDirty
.globl BGNameList
.globl BGListIndex
.globl BGPatternCache

.globl LUT
.globl BPLUT
.globl Render_Palette
.globl SMSColorTable
.globl GGColorTable

.globl AtexMask

.globl Render_UpdateBgCache
.globl Render_SMSBgLine

.globl VDP

.data
.align 4

BGNameDirty:    .space 0x200
BGNameList:     .space 0x200 * 2
BGListIndex:    .space 16
BGPatternCache: .space 0x20000

LUT:            .space 0x10000
BPLUT:          .space 0x10000 * 4
Render_Palette: .space 0x100 * 2
SMSColorTable:  .space 0x100 * 2
GGColorTable:   .space 0x100 * 4

i:              .space 16
AtexMask:       .word 0x00000000, 0x10101010, 0x20202020, 0x30303030

.text
.align 4

# ******************************************************************************************
# void Render_UpdateBgCache(void)

Render_UpdateBgCache:
  
  lw    $t0, BGListIndex
  beqz  $t0, ubc_end
  
  sll   $t0, 1
  la    $t1, BGNameList
  addu  $t0, $t1
  sw    $t0, i

  ubc_loop1:
    lhu   $t3, 0($t1)
    sh    $0, 0($t1)

    la    $t5, BGNameDirty
    addu  $t5, $t3
    
    move  $t4, $0
    ubc_loop2:
      li    $t6, 1
      sll   $t6, $t4
      lbu   $t7, 0($t5)
      and   $t6, $t7
      beqz  $t6, ubc_next
      
      la    $t8, BGPatternCache
      sll   $t9, $t3, 6
      addu  $t7, $t8, $t9
      
      la    $t6, VDP
      sll   $t8, $t4, 2
      sll   $t9, $t3, 5
      or    $t9, $t8
      addu  $t6, $t9
      lhu   $t8, 0($t6)
      lhu   $t9, 2($t6)
      sll   $t8, 2
      sll   $t9, 2
      la    $t6, BPLUT
      addu  $t8, $t6
      addu  $t9, $t6
      lw    $t8, 0($t8)
      lw    $t9, 0($t9)
      srl   $t8, 2
      or    $t8, $t9

      move  $t6, $0
      ubc_loop3:
        sll   $t0, $t6, 2
        srl   $t9, $t8, $t0
        and   $t9, 0x0F
        
        sll   $t2, $t4, 3
        or    $t0, $t2, $t6
        addu  $t0, $t7
        sb    $t9, 0($t0)
        
        xor   $t0, $t6, 7
        or    $t2, $t0
        or    $t2, 0x08000
        addu  $t2, $t7
        sb    $t9, 0($t2)
        
        xor   $t2, $t4, 7
        sll   $t2, 3
        or    $t0, $t2
        or    $t0, 0x18000
        addu  $t0, $t7
        sb    $t9, 0($t0)

        or    $t2, $t6
        or    $t2, 0x10000
        addu  $t2, $t7
        sb    $t9, 0($t2)
        
        addu  $t6, 1
        bltu  $t6, 8, ubc_loop3

      ubc_next:
      addu  $t4, 1
      bltu  $t4, 8, ubc_loop2
    
    sb    $0, 0($t5)
    
    addu  $t1, 2
    lw    $t0, i
    bltu  $t1, $t0, ubc_loop1

  sw    $0, BGListIndex
  
  ubc_end:
  jr    $ra
# ******************************************************************************************



# ******************************************************************************************
# void Render_SMSBgLine(unsigned char *linebuf)

.macro sbl_inter_step
    addu  $t6, $t1, $t3
    and   $t6, 0x1F
    sll   $t6, 1
    addu  $t6, $t4
    lhu   $t5, 0($t6)              # $t5 = attr
    
    srl   $t6, $t5, 11
    and   $t6, 0x03
    sll   $t6, 2
    la    $t7, AtexMask
    addu  $t7, $t6
    lw    $t6, 0($t7)              # $t6 = atex_mask
    
    and   $t5, 0x7FF
    sll   $t5, 6
    or    $t5, $t0
    la    $t7, BGPatternCache
    add   $t5, $t7                 # $t5 = cache_ptr
    
    sll   $t7, $t3, 3
    add   $t7, $t2                 # $t7 = dst
    
    ld    $t8, 0($t5)
    or    $t8, $t6
    or    $t9, $t6
.endm

.macro sbl_step_aligned max
  __loop_aligned_\max:
    sbl_inter_step
    sd    $t8, 0($t7)
    addu  $t3, 1                   # $t3 = column
    bltu  $t3, \max, __loop_aligned_\max
.endm

.macro sbl_step_unaligned max
  __loop_unaligned_\max:
    sbl_inter_step
    usw   $t8, 0($t7)
    usw   $t9, 4($t7)    
    addu  $t3, 1                   # $t3 = column
    bltu  $t3, \max, __loop_unaligned_\max
.endm

.macro sbl_check_reg0 label
  # if(VDP.Reg[0] & 0x80)
  lbu   $t5, (VDP + 0x4040)
  and   $t5, 0x80
  beqz  $t5, \label
  lw    $t0, (VDP + 0x4060)
  and   $t0, 0x07
  sll   $t0, 3                     # $t0 = v_row
  lbu   $t4, (VDP + 0x4042)
  sll   $t4, 10
  and   $t4, 0x3800
  lw    $t5, (VDP + 0x4060)
  srl   $t5, 3
  sll   $t5, 6
  add   $t4, $t5
  la    $t5, VDP
  add   $t4, $t5                   # $t4 = nt
.endm

.align 4

# void Render_SMSBgLine(unsigned char *linebuf)
Render_SMSBgLine:

  # $t0 = VDP.Line
  lw    $t0, (VDP + 0x4060)

  # $t1 = hscroll
  lbu   $t2, (VDP + 0x4048)
  li    $t1, 0x100
  subu  $t1, $t2
  lbu   $t2, (VDP + 0x4040)
  and   $t2, 0x40
  beqz  $t2, sbl_scroll
  bge   $t0, 0x10, sbl_scroll
  move  $t1, $0
  sbl_scroll:

  # $t2 = shift
  and   $t2, $t1, 0x07
  move  $t3, $0
  
  # if(shift)
  beqz  $t2, sbl_noshift
  move  $t3, $a0
  li    $t4, 8
  subu  $t4, $t2
  addu  $t4, $t3
  sbl_loop1:
    sb    $0, 0($t3)
    addu  $t3, 1
    bltu  $t3, $t4, sbl_loop1    
  li    $t3, 1                     # $t3 = column
  sbl_noshift:

  sw    $t2, i                     # i   = shift
  subu  $t2, $a0, $t2              # $t2 = linebuf_ptr
  srl   $t1, 3                     # $t1 = nt_scroll
  lbu   $t6, (VDP + 0x4049)
  addu  $t5, $t0, $t6
  remu  $t4, $t5, 224              # $t4 = v_line
  and   $t0, $t4, 0x07
  sll   $t0, 3                     # $t0 = v_row
  lw    $t6, (VDP + 0x4058)
  srl   $t4, 3
  sll   $t4, 6
  addu  $t4, $t6
  la    $t6, VDP
  addu  $t4, $t6                   # $t4 = nt
  
  # if((((unsigned)linebuf_ptr) & 0x03) == 0)
  and   $t5, $t2, 0x03
  beqz  $t5, sbl_memalign32

  # unaligned address
  sbl_step_unaligned 24
  sbl_check_reg0 sbl_next1
  sbl_next1:
  sbl_step_unaligned 32
  j     sbl_next3

  # aligned address
  .align 4
  sbl_memalign32:
  sbl_step_aligned 24
  sbl_check_reg0 sbl_next2
  sbl_next2:
  sbl_step_aligned 32

  sbl_next3:
  lw    $t5, i                     # $t5 = shift
  beqz  $t5, sbl_end               # if(shift)
  
  sll   $t6, $t3, 3
  subu  $t6, $t5
  addu  $t6, $a0                   # $t6 = p
  addu  $t7, $t3, $t1
  and   $t7, 0x1F
  sll   $t7, 1
  addu  $t7, $t4
  lhu   $t3, 0($t7)                # $t3 = attr
  srl   $t7, $t3, 7
  and   $t7, 0x30                  # $t7 = a
  la    $t2, BGPatternCache
  and   $t3, 0x7FF
  sll   $t3, 6
  or    $t3, $t0
  addu  $t2, $t3                   # $t2 = &BGPatternCache[((attr & 0x7FF) << 6) | v_row]
  move  $t4, $0                    # $t4 = x
  sbl_loop2:
    or    $t1, $t2, $t4
    lbu   $t0, 0($t1)
    or    $t0, $t7                 # $t0 = c | a
    sb    $t0, 0($t6)
    add   $t6, 1
    add   $t4, 1
    blt   $t4, $t5, sbl_loop2
  
  sbl_end:
  jr    $ra
# ******************************************************************************************

.align 4
nop
nop
nop
nop
