/* script.h - this file is part of segax project. Rodrigo Cardoso.
** Script file menagement. */

#ifndef _SCRIPT_H
#define _SCRIPT_H

#include <stdio.h>
#include "def.h" // only for BOOL type

/* if SCRIPT_USE_TEMP_FILE is defined, uses a temporary file to store string values
   instead of RAM, this can reduce the amount of RAM used to store script files. */
#define SCRIPT_USE_TEMP_FILE

// script data types
typedef enum _SCRIPT_DATA_TYPE {
  SDT_NULL,
  SDT_STRING,
  SDT_NUMBER,
  SDT_BOOL,
  SDT_MULTI
} SCRIPT_DATA_TYPE;

typedef struct _SCRIPT_DATA {
  char            *name;
  int              name_size;
  union {
#ifdef SCRIPT_USE_TEMP_FILE
    long           data_pos;
#else
    char          *sz_data;
    int           *pn_data;
#endif
    int            n_data;
  };
  long             data_size;
  SCRIPT_DATA_TYPE data_type;
} SCRIPT_DATA;

typedef struct _SCRIPT_FILE {
#ifdef SCRIPT_USE_TEMP_FILE
  void        *tmp_file;   // temporary file used to store strings and multiple numbers
#endif
  int          n_alloc;    // number of pointers in data
  int          n_data;     // total number of parameters
  SCRIPT_DATA *data;       // pointer to parameters
  char        *names_base; // memory block to hold parameters name
  int          base_size;  // total size of the memory pointed by names base
  int          names_size; // size of the memory used by the names
} SCRIPT_FILE;


#ifdef __cplusplus
extern "C" {
#endif

SCRIPT_FILE *Script_Create(void);        // create a new script file on memory
void         Script_Free(SCRIPT_FILE *); // free the memory used to store the script file
SCRIPT_FILE *Script_Load(const char *);  // load the script file into memory
BOOL         Script_Save(SCRIPT_FILE *, const char *, const char *); // save the script on memory into a file
/* load a string from the script file, return the size of the string (including the null
** terminate character) */
int          Script_GetString(SCRIPT_FILE *, const char *, const char *, int, char *, int, BOOL *);
int          Script_GetInt(SCRIPT_FILE *, const char *, int, BOOL *);
BOOL         Script_GetBool(SCRIPT_FILE *, const char *, BOOL , BOOL *);
int          Script_GetMultiInts(SCRIPT_FILE *, const char *, const int *, int *, int, BOOL *);
int          Script_GetStringSize(SCRIPT_FILE *, const char *);    // return the size of the string, or -1 if string does not exist
int          Script_GetNumberOfInts(SCRIPT_FILE *, const char *);  // return the number of integers or -1 if does not exist
BOOL         Script_WriteString(SCRIPT_FILE *, const char *, const char *, int);
BOOL         Script_WriteInt(SCRIPT_FILE *, const char *, int);
BOOL         Script_WriteBool(SCRIPT_FILE *, const char *, BOOL);
BOOL         Script_WriteMultiInts(SCRIPT_FILE *, const char *, const int *, int);
BOOL         Script_WriteStringDirect(FILE *, const char *, const char *, int, const char *, int);
BOOL         Script_WriteIntDirect(FILE *, const char *, int, const char *, int);
BOOL         Script_WriteBoolDirect(FILE *, const char *, BOOL, const char *, int);
BOOL         Script_WriteMultiIntsDirect(FILE *pFile, const char *, const int *, int, const char *, int);

#ifdef __cplusplus
}
#endif

#endif // _GSCRIPT_H
