
#include <stdio.h>
#include <string.h>
#include "png.h"
#include "shot.h"
#include "video.h"
#include "system.h"
#include "def.h"

typedef struct _BMPHDR {
  unsigned long  bfSize;
  unsigned short bfReserved1;
  unsigned short bfReserved2;
  unsigned long  bfOffBits;
  unsigned long  biSize;
  long           biWidth;
  long           biHeight;
  unsigned short biPlanes;
  unsigned short biBitCount;
  unsigned long  biCompression;
  unsigned long  biSizeImage;
  long           biXPelsPerMeter;
  long           biYPelsPerMeter;
  unsigned long  biClrUsed;
  unsigned long  biClrImportant;
} BMPHDR;

BOOL Shot_SaveBMP(const char *file)
{
  FILE               *f;
  BMPHDR              hdr;
  unsigned char      *src;
  int                 n, m;
  int                 x, y, sx, sy;
  const VIDEO_BUFFER *buf;

  f = fopen(file, "wb");
  if(!f) return FALSE;

  if(System.Flags & SF_GAME_GEAR) { x = 48; sx = 160; y = 24; sy = 144; }
  else                            { x = 0;  sx = 256; y = 0;  sy = 192; }

  if((putc('B', f) == EOF) || (putc('M', f) == EOF)) return fclose(f), FALSE;
  memset(&hdr, 0, sizeof(BMPHDR));
  hdr.bfOffBits  = sizeof(BMPHDR) + 2 + 1024;
  hdr.biSize     = 40;
  hdr.biBitCount = 8;
  hdr.biPlanes   = 1;
  hdr.biWidth    = sx;
  hdr.biHeight   = sy;
  if(fwrite(&hdr, sizeof(BMPHDR), 1, f) != 1) return fclose(f), FALSE;
  for(n = 0; n < 256; n++)
  {
    if(putc(((Render_Palette[n] >> 8) & 0xF8), f) == EOF) return fclose(f), FALSE;
    if(putc(((Render_Palette[n] >> 3) & 0xF8), f) == EOF) return fclose(f), FALSE;
    if(putc(((Render_Palette[n] << 3) & 0xF8), f) == EOF) return fclose(f), FALSE;
    if(putc(0, f) == EOF) return fclose(f), FALSE;
  }
  buf = Video_GetScreenBuffer();
  src = (unsigned char *)buf->Data + x + (y * buf->Pitch);
  m   = sy - 1;
  for(n = 0; n < sy; n++)
  {
    if(fwrite(src + ((m - n) * buf->Pitch), sx, 1,  f) != 1) return fclose(f), FALSE;
  }
  fclose(f);

  return TRUE;
}

BOOL Shot_SavePNG(const char *szFile)
{
  FILE               *fp;
  png_structp         png_ptr;
  png_infop           info_ptr;
  png_color           palette[256];
  int                 i, j, num_colors, change, depth;
  unsigned char       buf[256], buf2[256], *line, *out;
  unsigned char       replace[256], used[256];
  int                 x, y, sx, sy;
  int                 depth_mask[8] = { 1, 2, 4, 4, 8, 8, 8, 8 };
  const VIDEO_BUFFER *src;

  // open the file
  fp = fopen(szFile, "wb");
  if(!fp) return FALSE;

  // image size
  if(System.Flags & SF_GAME_GEAR) { x = 48; sx = 160; y = 24; sy = 144; }
  else                            { x = 0;  sx = 256; y = 0;  sy = 192; }

  // create color palette
  for(i = 0; i < 256; i++)
  {
    palette[i].red   = (Render_Palette[i] << 3) & 0xF8;
    palette[i].green = (Render_Palette[i] >> 3) & 0xFC;
    palette[i].blue  = (Render_Palette[i] >> 8) & 0xF8;
  }

  // check if there are duplicated colors in palette
  for(i = 0; i < 256; i++) { replace[i] = (unsigned char)i; used[i] = 1; }
  for(i = 0; i < 255; i++)
  {
    if(used[i])
    {
      for(j = i + 1; j < 256; j++)
      {
        if(used[j] && Render_Palette[i] == Render_Palette[j])
        { replace[j] = (unsigned char)i; used[j] = 0; }
      }
    }
  }

  // check if there are any unused color in palette
  for(i = 0; i < 256; i++) used[i] = 0;
  src = Video_GetScreenBuffer();
  line = (unsigned char *)src->Data + x + (y * src->Pitch);
  for(j = 0; j < sy; j++)
  {
    for(i = 0; i < sx; i++) used[replace[line[i]]] = 1;
    line += src->Pitch;
  }

  // move unused colors to the end of the palette
  do
  {
    change = 0;
    for(i = 0; i < 255; i++)
    {
      if(used[i] == 0 && used[i + 1] == 1)
      {
        palette[i].red   = palette[i + 1].red;
        palette[i].green = palette[i + 1].green;
        palette[i].blue  = palette[i + 1].blue;
        used[i]   = 1;
        used[++i] = 0;
        for(j = 0; j < 256; j++) { if(replace[j] == i) replace[j]--; }
        i--;
        change = 1;
      }
    }
  } while(change);
  
  // fill unused colors in palette with black
  for(i = 0; i < 256; i++)
  { if(!used[i]) palette[i].red = palette[i].green = palette[i].blue = 0; }

  // counting number of colors
  for(j = 0; j < 256; j++) { if(!used[j]) break; }
  for(i = 0, num_colors = 1; i < 8; i++) { num_colors <<= 1; if(j <= num_colors) break; }
  depth = depth_mask[i];

  // png information
  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if(!png_ptr) return fclose(fp), FALSE;
  info_ptr = png_create_info_struct(png_ptr);
  if(!info_ptr) return png_destroy_write_struct(&png_ptr, NULL), fclose(fp), FALSE;
  if(setjmp(png_jmpbuf(png_ptr))) return png_destroy_write_struct(&png_ptr, &info_ptr), fclose(fp), FALSE;
  png_init_io(png_ptr, fp);
  png_set_IHDR(png_ptr, info_ptr, sx, sy, depth, PNG_COLOR_TYPE_PALETTE,
               PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
  png_set_PLTE(png_ptr, info_ptr, palette, num_colors);
  png_write_info(png_ptr, info_ptr);

  // saving image
  out  = buf;
  line = (unsigned char *)src->Data + x + (y * src->Pitch);
  for(i = 0; i < sy; i++)
  {
    for(j = 0; j < sx; j++) buf[j] = replace[line[j]];
    switch(depth)
    {
      case 4:
        out = buf2;
        for(j = 0; j < sx; j += 2) buf2[j >> 1] = buf[j + 1] | (buf[j] << 4);
        break;
      case 2:
        out = buf2;
        for(j = 0; j < sx; j += 4)
          buf2[j >> 2] = buf[j + 3] | (buf[j + 2] << 2) | (buf[j + 1] << 4) | (buf[j] << 6);
        break;
      case 1:
        out = buf2;
        for(j = 0; j < sx; j += 8)
          buf2[j >> 3] = buf[j + 7] | (buf[j + 6] << 1) | (buf[j + 5] << 2) | (buf[j + 4] << 3) |
                        (buf[j + 3] << 4) | (buf[j + 2] << 5) | (buf[j + 1] << 6) | (buf[j] << 7);
        break;
    }
    png_write_row(png_ptr, out);
    line += src->Pitch;
  }

  png_write_end(png_ptr, info_ptr);
  png_destroy_write_struct(&png_ptr, &info_ptr);
  fclose(fp);

  return TRUE;
}
