
#include <stdio.h>
#include <string.h>
#include "text.h"
#include "video.h"
#include "def.h"

extern unsigned long Font1_Color[2];
extern unsigned long Font1_DefColor[2];
extern unsigned char Font1_Data[2][1024];
extern unsigned long Font2_Color;
extern unsigned long Font2_DefColor;
extern unsigned char Font2_Data[1024];
extern unsigned long Font3_Color;
extern unsigned long Font3_DefColor;
extern unsigned char Font3_Data[512];

#define FONTS_FILE_TYPE    0x004E4647
#define FONT_NAME_MAX_SIZE 32

typedef struct _FONT_INFO {
  char          pszName[FONT_NAME_MAX_SIZE]; // font name
  int           iType;                       // font tipe (1, 2 ou 3)
  unsigned      uFileAdr;                    // font address in the fonts file
  unsigned long c1;                          // font default color 1
  unsigned long c2;                          // font default color 2
} FONT_INFO;

BOOL Text_LoadFont(const char *fonts_file, const char *font_name, int type)
{
  FILE         *f;
  unsigned long n[2], i;
  FONT_INFO     fi;

  f = fopen(fonts_file, "rb");
  if(!f) return FALSE;
  if(fread(n, 8, 1, f) != 1)                 return fclose(f), FALSE;
  if(n[0] != FONTS_FILE_TYPE || n[1] > 1024) return fclose(f), FALSE;
  for(i = 0; i < n[1]; i++)
  {
    if(fread(&fi, sizeof(FONT_INFO), 1, f) != 1) return fclose(f), FALSE;
    if(type == fi.iType && !strcmpi(font_name, fi.pszName))
    {
      if(fseek(f, fi.uFileAdr, SEEK_SET)) return fclose(f), FALSE;
      switch(fi.iType)
      {
        case 1:
          if(fread(Font1_Data, 2048, 1, f) != 1) return fclose(f), FALSE;
          Font1_DefColor[0] = Font1_Color[0] = fi.c2 & 0xFFFFFF;
          Font1_DefColor[1] = Font1_Color[1] = fi.c1 & 0xFFFFFF;
          break;
        case 2:
          if(fread(Font2_Data, 1024, 1, f) != 1) return fclose(f), FALSE;
          Font2_DefColor = (unsigned short)(((fi.c1 & 0x0000F8) >> 3) | ((fi.c1 & 0x00FC00) >> 5) | ((fi.c1 & 0xF80000) >> 8));
          Font2_DefColor = Font2_Color = fi.c1 & 0xFFFFFF;
          break;
        case 3:
          if(fread(Font3_Data, 512, 1, f) != 1) return fclose(f), FALSE;
          Font3_DefColor = (unsigned short)(((fi.c1 & 0x0000F8) >> 3) | ((fi.c1 & 0x00FC00) >> 5) | ((fi.c1 & 0xF80000) >> 8));
          Font3_DefColor = Font3_Color = fi.c1 & 0xFFFFFF;
          break;
        default:
          return fclose(f), FALSE;
      }
      return fclose(f), TRUE;
    }
  }
  fclose(f);

  return FALSE;
}

void Text_SetFontColor(int font_type, unsigned long c1, unsigned long c2)
{
  switch(font_type)
  {
    case 1:
      if(c2 & 0xFF000000) Font1_Color[0] = Font1_DefColor[0];
      else                Font1_Color[0] = c2;
      if(c1 & 0xFF000000) Font1_Color[1] = Font1_DefColor[1];
      else                Font1_Color[1] = c1;
    case 2:
      if(c1 & 0xFF000000) Font2_Color = Font2_DefColor;
      else                Font2_Color = c1;
      break;
    case 3:
      if(c1 & 0xFF000000) Font3_Color = Font2_DefColor;
      else                Font3_Color = c1;
      break;
  }
}

void Text_InvertFontColor(int font_type)
{
  switch(font_type)
  {
    case 1:
      Font1_Color[0] = (~Font1_Color[0]) & 0xFFFFFF;
      Font1_Color[1] = (~Font1_Color[1]) & 0xFFFFFF;
      break;
    case 2:
      Font2_Color = (~Font2_Color) & 0xFFFFFF;
      break;
    case 3:
      Font3_Color = (~Font3_Color) & 0xFFFFFF;
      break;
  }
}

#define _DRAW1(Z, C)          \
  if(c & 0x80##Z) dst[0] = C; \
  if(c & 0x40##Z) dst[1] = C; \
  if(c & 0x20##Z) dst[2] = C; \
  if(c & 0x10##Z) dst[3] = C; \
  if(c & 0x08##Z) dst[4] = C; \
  if(c & 0x04##Z) dst[5] = C; \
  if(c & 0x02##Z) dst[6] = C; \
  if(c & 0x01##Z) dst[7] = C;

#define _DRAW2(C)          \
  if(c & 0x80) dst[0] = C; \
  if(c & 0x40) dst[1] = C; \
  if(c & 0x20) dst[2] = C; \
  if(c & 0x10) dst[3] = C; \
  if(c & 0x08) dst[4] = C; \
  if(c & 0x04) dst[5] = C; \
  if(c & 0x02) dst[6] = C; \
  if(c & 0x01) dst[7] = C;

#define _DRAW3(Z, C)         \
  if(c & 0x8##Z) dst[0] = C; \
  if(c & 0x4##Z) dst[1] = C; \
  if(c & 0x2##Z) dst[2] = C; \
  if(c & 0x1##Z) dst[3] = C; \

#define _DRAW4(C)         \
  if(c & 0x8) dst[0] = C; \
  if(c & 0x4) dst[1] = C; \
  if(c & 0x2) dst[2] = C; \
  if(c & 0x1) dst[3] = C; \

#define _DRAW_CHAR1(T, C)                                                       \
  dst = (T *)((unsigned char *)buf->Data + (x * sizeof(T)) + (y * buf->Pitch)); \
  for(j = 0; j < 2; src--, dst = (T *)((unsigned char *)dst + buf->Pitch), j++) \
  {                                                                             \
    c = *src;                                                                   \
    _DRAW1(000000, C)                                                           \
    dst = (T *)((unsigned char *)dst + buf->Pitch);                             \
    _DRAW1(0000, C)                                                             \
    dst = (T *)((unsigned char *)dst + buf->Pitch);                             \
    _DRAW1(00, C)                                                               \
    dst = (T *)((unsigned char *)dst + buf->Pitch);                             \
    _DRAW2(C)                                                                   \
  }

#define _DRAW_CHAR2(T, C)                                                       \
  dst = (T *)((unsigned char *)buf->Data + (x * sizeof(T)) + (y * buf->Pitch)); \
  _DRAW3(0000000, C)                                                            \
  dst = (T *)((unsigned char *)dst + buf->Pitch);                               \
  _DRAW3(000000, C)                                                             \
  dst = (T *)((unsigned char *)dst + buf->Pitch);                               \
  _DRAW3(00000, C)                                                              \
  dst = (T *)((unsigned char *)dst + buf->Pitch);                               \
  _DRAW3(0000, C)                                                               \
  dst = (T *)((unsigned char *)dst + buf->Pitch);                               \
  _DRAW3(000, C)                                                                \
  dst = (T *)((unsigned char *)dst + buf->Pitch);                               \
  _DRAW3(00, C)                                                                 \
  dst = (T *)((unsigned char *)dst + buf->Pitch);                               \
  _DRAW3(0, C)                                                                  \
  dst = (T *)((unsigned char *)dst + buf->Pitch);                               \
  _DRAW4(C)

void Text_Draw(int font_type, const char *text, int x, int y, unsigned long flags)
{
  int                 j;
  unsigned long      *src;
  unsigned long       c;
  int                 n;
  const VIDEO_BUFFER *buf;

  if(!text) return;

  if(flags & TDF_RIGHT)  x -= strlen(text) * (font_type == 3 ? 4 : 8);
  if(flags & TDF_CENTER) x -= (strlen(text) * (font_type == 3 ? 4 : 8)) >> 1;
  if(flags & TDF_CENTER_SCREEN_X)
  {
    int n = (font_type == 3 ? 4 : 8);
    x = (480 - (strlen(text) * n)) >> 1; // center x
  }
  if(flags & TDF_CENTER_SCREEN_Y) y = (272 - 8) >> 1;

  if(y < 0 || y > 264) return;

  if(font_type == 1)
  {
    if(Video_GetMode() == VM_EMULATION) // 16 bits color
    {
      unsigned short *dst;
      unsigned short  clr[2];

      buf = Video_GetDrawBuffer();
      clr[0] = (unsigned short)(((Font1_Color[0] & 0x0000F8) >> 3) | ((Font1_Color[0] & 0x00FC00) >> 5) | ((Font1_Color[0] & 0xF80000) >> 8));
      clr[1] = (unsigned short)(((Font1_Color[1] & 0x0000F8) >> 3) | ((Font1_Color[1] & 0x00FC00) >> 5) | ((Font1_Color[1] & 0xF80000) >> 8));

      while((n = *text))
      {
        if(n & 0x80) n = '?'; // invalid character
        if     (x > 472) return;
        else if(x >= 0)
        {
          src = (unsigned long *) (Font1_Data[0] + (n * 8) + 4);
          _DRAW_CHAR1(unsigned short, clr[0])
          src = (unsigned long *) (Font1_Data[1] + (n * 8) + 4);
          _DRAW_CHAR1(unsigned short, clr[1])
        }
        text++;
        x += 8;
      }
    }
    else if(Video_GetMode() == VM_MENU) // 32 bits color
    {
      unsigned long *dst;

      buf = Video_GetDrawBuffer();

      while((n = *text))
      {
        if(n & 0x80) n = '?'; // invalid character
        if     (x > 472) return;
        else if(x >= 0)
        {
          src = (unsigned long *) (Font1_Data[0] + (n * 8) + 4);
          _DRAW_CHAR1(unsigned long, Font1_Color[0])
          src = (unsigned long *) (Font1_Data[1] + (n * 8) + 4);
          _DRAW_CHAR1(unsigned long, Font1_Color[1])
        }
        text++;
        x += 8;
      }
    }
  }
  else if(font_type == 2)
  {
    if(Video_GetMode() == VM_EMULATION) // 16 bits color
    {
      unsigned short *dst;
      unsigned short  clr;

      buf = Video_GetDrawBuffer();
      clr = (unsigned short)(((Font2_Color & 0x0000F8) >> 3) | ((Font2_Color & 0x00FC00) >> 5) | ((Font2_Color & 0xF80000) >> 8));

      while((n = *text))
      {
        if(n & 0x80) n = '?'; // invalid character
        if     (x > 472) return;
        else if(x >= 0)
        {
          src = (unsigned long *) (Font2_Data + (n * 8) + 4);
          _DRAW_CHAR1(unsigned short, clr)
        }
        text++;
        x += 8;
      }
    }
    else if(Video_GetMode() == VM_MENU) // 32 bits color
    {
      unsigned long  *dst;

      buf = Video_GetDrawBuffer();

      while((n = *text))
      {
        if(n & 0x80) n = '?'; // invalid character
        if     (x > 472) return;
        else if(x >= 0)
        {
          src = (unsigned long *) (Font2_Data + (n * 8) + 4);
          _DRAW_CHAR1(unsigned long, Font2_Color)
        }
        text++;
        x += 8;
      }
    }
  }
  else if(font_type == 3)
  {
    if(Video_GetMode() == VM_EMULATION) // 16 bits color
    {
      unsigned short *dst;
      unsigned short  clr;

      buf = Video_GetDrawBuffer();
      clr = (unsigned short)(((Font3_Color & 0x0000F8) >> 3) | ((Font3_Color & 0x00FC00) >> 5) | ((Font3_Color & 0xF80000) >> 8));

      while((n = *text))
      {
        if(n & 0x80) n = '?'; // invalid character
        if     (x > 476) return;
        else if(x >= 0)
        {
          c = *(unsigned long *)(Font3_Data + (n * 4));
          _DRAW_CHAR2(unsigned short, clr)
        }
        text++;
        x += 4;
      }
    }
    else if(Video_GetMode() == VM_MENU) // 32 bits color
    {
      unsigned long *dst;

      buf = Video_GetDrawBuffer();

      while((n = *text))
      {
        if(n & 0x80) n = '?'; // invalid character
        if     (x > 476) return;
        else if(x >= 0)
        {
          c = *(unsigned long *)(Font3_Data + (n * 4));
          _DRAW_CHAR2(unsigned long, Font3_Color)
        }
        text++;
        x += 4;
      }
    }
  }
}

unsigned long Font1_Color[2]    = { 0x000000, 0xFFFFFF };
unsigned long Font1_DefColor[2] = { 0x000000, 0xFFFFFF };

unsigned char ALIGN(4) Font1_Data[2][1024] = { {
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x00
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x01
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x02
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x03
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x04
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x05
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x06
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x07
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0x08
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // tab	
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // line break
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x0A
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x0B
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x0C
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // line break
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x0E
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x0F
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x10
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x11
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x12
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x13
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x14
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x15
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x16
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x17
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x18
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x19
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x1A
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // space
  0x3C, 0x24, 0x3C, 0x24, 0x24, 0x24, 0x24, 0x3C, // !
  0x00, 0x00, 0x00, 0x22, 0x66, 0xAA, 0xAA, 0xEE, // "
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // #
  0xEC, 0x86, 0xEA, 0xC6, 0xAE, 0xC2, 0x6E, 0x38, // $
  0x67, 0x55, 0x2B, 0x14, 0xCA, 0xA5, 0xE3, 0x00, // %
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // &
  0x00, 0x00, 0x00, 0x08, 0x18, 0x28, 0x28, 0x38, // 
  0x0E, 0x1A, 0x14, 0x14, 0x14, 0x14, 0x1A, 0x0E, // (
  0x70, 0x58, 0x28, 0x28, 0x28, 0x28, 0x58, 0x70, // )
  0x00, 0xC6, 0xFE, 0x6C, 0x44, 0xC6, 0xEE, 0x38, // *
  0x38, 0x28, 0xEE, 0x82, 0xEE, 0x28, 0x38, 0x00, // +
  0x30, 0x18, 0x28, 0x28, 0x38, 0x00, 0x00, 0x00, // ,
  0x00, 0x00, 0x00, 0xFE, 0x82, 0xFE, 0x00, 0x00, // -
  0x00, 0x38, 0x28, 0x38, 0x00, 0x00, 0x00, 0x00, // .
  0xE0, 0xB0, 0xD8, 0x6C, 0x36, 0x1B, 0x0D, 0x07, // /
  0x7E, 0xC3, 0x99, 0x99, 0x99, 0x99, 0xC3, 0x7E, // 0
  0x7E, 0x42, 0x66, 0x24, 0x64, 0x44, 0x64, 0x3C, // 1
  0xFF, 0x81, 0x9F, 0xC3, 0xF9, 0x99, 0xC3, 0x7E, // 2
  0x7E, 0xC3, 0x99, 0xF9, 0xF3, 0x99, 0xC3, 0x7E, // 3
  0x1E, 0xF3, 0x81, 0xB3, 0x92, 0xC2, 0x62, 0x3E, // 4
  0xFE, 0x83, 0xF9, 0xF9, 0x83, 0x9F, 0x81, 0xFF, // 5
  0x7E, 0xC3, 0x99, 0x99, 0x83, 0x9E, 0xC2, 0x7E, // 6
  0x3C, 0x24, 0x24, 0x26, 0x33, 0xF9, 0x81, 0xFF, // 7
  0x7E, 0xC3, 0x99, 0x99, 0xC3, 0x99, 0xC3, 0x7E, // 8
  0x3E, 0x23, 0x79, 0xC9, 0x99, 0x99, 0xC3, 0x7E, // 9
  0x38, 0x28, 0x38, 0x00, 0x38, 0x28, 0x38, 0x00, // :
  0x30, 0x18, 0x28, 0x38, 0x00, 0x38, 0x28, 0x38, // ;
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // <
  0x00, 0xFE, 0x82, 0xFE, 0xFE, 0x82, 0xFE, 0x00, // =
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // >
  0x3C, 0x24, 0x3C, 0x26, 0xF2, 0x92, 0xE6, 0x7C, // ?
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // @
  0xFF, 0x99, 0x99, 0x81, 0x99, 0x99, 0xC3, 0x7E, // A
  0xFE, 0x83, 0x99, 0x99, 0x83, 0x99, 0x83, 0xFE, // B
  0x7E, 0xC3, 0x99, 0x9F, 0x9F, 0x99, 0xC3, 0x7E, // C
  0xFE, 0x83, 0x99, 0x99, 0x99, 0x99, 0x83, 0xFE, // D
  0xFF, 0x81, 0x9F, 0x9E, 0x82, 0x9F, 0x81, 0xFF, // E
  0xF0, 0x90, 0x90, 0x9E, 0x82, 0x9F, 0x81, 0xFF, // F
  0x7F, 0xC1, 0x99, 0x99, 0x91, 0x9F, 0xC1, 0x7F, // G
  0xFF, 0x99, 0x99, 0x99, 0x81, 0x99, 0x99, 0xFF, // H
  0x7E, 0x42, 0x66, 0x24, 0x24, 0x66, 0x42, 0x7E, // I
  0x78, 0xE4, 0xA4, 0xE4, 0x64, 0x66, 0x42, 0x7E, // J
  0xFE, 0x92, 0x92, 0x86, 0x86, 0x93, 0x99, 0xFF, // K
  0xFF, 0x81, 0x9F, 0x90, 0x90, 0x90, 0x90, 0xF0, // L
  0xFF, 0x99, 0x99, 0x81, 0x81, 0x99, 0xBD, 0xE7, // M
  0xFF, 0x91, 0x91, 0x81, 0x81, 0x89, 0x99, 0xFF, // N
  0x7E, 0xC3, 0x99, 0x99, 0x99, 0x99, 0xC3, 0x7E, // O
  0xF0, 0x90, 0x9E, 0x83, 0x99, 0x99, 0x83, 0xFE, // P
  0x7E, 0xD1, 0x97, 0x99, 0x99, 0x99, 0xC3, 0x7E, // Q
  0xFF, 0x99, 0x99, 0x83, 0x99, 0x99, 0x83, 0xFE, // R
  0xFE, 0x83, 0xF9, 0xC1, 0x83, 0x9F, 0xC1, 0x7F, // S
  0x3C, 0x24, 0x24, 0x24, 0x24, 0xE7, 0x81, 0xFF, // T
  0x7E, 0xC3, 0x99, 0x99, 0x99, 0x99, 0x99, 0xFF, // U
  0x3C, 0x66, 0xDB, 0x99, 0x99, 0x99, 0x99, 0xFF, // V
  0x7E, 0xDB, 0x81, 0x81, 0x99, 0x99, 0x99, 0xFF, // W
  0xFF, 0x99, 0xCB, 0x66, 0x66, 0xD3, 0x99, 0xFF, // X
  0x3C, 0x24, 0x24, 0x66, 0xC3, 0x99, 0x99, 0xFF, // Y
  0xFF, 0x81, 0x8F, 0xC6, 0x63, 0xF1, 0x81, 0xFF, // Z
  0x1E, 0x12, 0x16, 0x14, 0x14, 0x16, 0x12, 0x1E, // [
  0x07, 0x0D, 0x1B, 0x36, 0x6C, 0xD8, 0xB0, 0xE0, // contra-barra
  0x78, 0x48, 0x68, 0x28, 0x28, 0x68, 0x48, 0x78, // ]
  0x00, 0x00, 0x00, 0x00, 0x7E, 0x5A, 0x66, 0x3C, // ^
  0xFF, 0x81, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, // _
  0x00, 0x00, 0x00, 0x00, 0x38, 0x68, 0x58, 0x70, // `
  0xFF, 0x99, 0x99, 0x81, 0x99, 0x99, 0xC3, 0x7E, // a
  0xFE, 0x83, 0x99, 0x99, 0x83, 0x99, 0x83, 0xFE, // b
  0x7E, 0xC3, 0x99, 0x9F, 0x9F, 0x99, 0xC3, 0x7E, // c
  0xFE, 0x83, 0x99, 0x99, 0x99, 0x99, 0x83, 0xFE, // d
  0xFF, 0x81, 0x9F, 0x9E, 0x82, 0x9F, 0x81, 0xFF, // e
  0xF0, 0x90, 0x90, 0x9E, 0x82, 0x9F, 0x81, 0xFF, // f
  0x7F, 0xC1, 0x99, 0x99, 0x91, 0x9F, 0xC1, 0x7F, // g
  0xFF, 0x99, 0x99, 0x99, 0x81, 0x99, 0x99, 0xFF, // h
  0x7E, 0x42, 0x66, 0x24, 0x24, 0x66, 0x42, 0x7E, // i
  0x78, 0xE4, 0xA4, 0xE4, 0x64, 0x66, 0x42, 0x7E, // j
  0xFE, 0x92, 0x92, 0x86, 0x86, 0x93, 0x99, 0xFF, // k
  0xFF, 0x81, 0x9F, 0x90, 0x90, 0x90, 0x90, 0xF0, // l
  0xFF, 0x99, 0x99, 0x81, 0x81, 0x99, 0xBD, 0xE7, // m
  0xFF, 0x91, 0x91, 0x81, 0x81, 0x89, 0x99, 0xFF, // n
  0x7E, 0xC3, 0x99, 0x99, 0x99, 0x99, 0xC3, 0x7E, // o
  0xF0, 0x90, 0x9E, 0x83, 0x99, 0x99, 0x83, 0xFE, // p
  0x7E, 0xD1, 0x97, 0x99, 0x99, 0x99, 0xC3, 0x7E, // q
  0xFF, 0x99, 0x99, 0x83, 0x99, 0x99, 0x83, 0xFE, // r
  0xFE, 0x83, 0xF9, 0xC1, 0x83, 0x9F, 0xC1, 0x7F, // s
  0x3C, 0x24, 0x24, 0x24, 0x24, 0xE7, 0x81, 0xFF, // t
  0x7E, 0xC3, 0x99, 0x99, 0x99, 0x99, 0x99, 0xFF, // u
  0x3C, 0x66, 0xDB, 0x99, 0x99, 0x99, 0x99, 0xFF, // v
  0x7E, 0xDB, 0x81, 0x81, 0x99, 0x99, 0x99, 0xFF, // w
  0xFF, 0x99, 0xCB, 0x66, 0x66, 0xD3, 0x99, 0xFF, // x
  0x3C, 0x24, 0x24, 0x66, 0xC3, 0x99, 0x99, 0xFF, // y
  0xFF, 0x81, 0x8F, 0xC6, 0x63, 0xF1, 0x81, 0xFF, // z
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // {
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // |
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // }
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // ~
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF, // 0x7F
}, {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x18, 0x00, 0x18, 0x18, 0x18, 0x18, 0x00, // !
  0x00, 0x00, 0x00, 0x00, 0x00, 0x44, 0x44, 0x00, // "
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // #
  0x10, 0x78, 0x14, 0x38, 0x50, 0x3C, 0x10, 0x00, // $
  0x00, 0x22, 0x10, 0x08, 0x04, 0x42, 0x00, 0x00, // %
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // &
  0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x00, // '
  0x00, 0x04, 0x08, 0x08, 0x08, 0x08, 0x04, 0x00, // (
  0x00, 0x20, 0x10, 0x10, 0x10, 0x10, 0x20, 0x00, // )
  0x00, 0x00, 0x00, 0x10, 0x38, 0x38, 0x10, 0x00, // *
  0x00, 0x10, 0x10, 0x7C, 0x10, 0x10, 0x00, 0x00, // +
  0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, // ,
  0x00, 0x00, 0x00, 0x00, 0x7C, 0x00, 0x00, 0x00, // -
  0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, // .
  0x00, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x00,
  0x00, 0x3C, 0x66, 0x66, 0x66, 0x66, 0x3C, 0x00, // 0
  0x00, 0x3C, 0x18, 0x18, 0x18, 0x38, 0x18, 0x00, // 1
  0x00, 0x7E, 0x60, 0x3C, 0x06, 0x66, 0x3C, 0x00, // 2
  0x00, 0x3C, 0x66, 0x06, 0x0C, 0x66, 0x3C, 0x00, // 3
  0x00, 0x0C, 0x7E, 0x4C, 0x6C, 0x3C, 0x1C, 0x00, // 4
  0x00, 0x7C, 0x06, 0x06, 0x7C, 0x60, 0x7E, 0x00, // 5
  0x00, 0x3C, 0x66, 0x66, 0x7C, 0x60, 0x3C, 0x00, // 6
  0x00, 0x18, 0x18, 0x18, 0x0C, 0x06, 0x7E, 0x00, // 7
  0x00, 0x3C, 0x66, 0x66, 0x3C, 0x66, 0x3C, 0x00, // 8
  0x00, 0x1C, 0x06, 0x36, 0x66, 0x66, 0x3C, 0x00, // 9
  0x00, 0x10, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, // :
  0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x10, 0x00, //, //
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // <
  0x00, 0x00, 0x7C, 0x00, 0x00, 0x7C, 0x00, 0x00, // =
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // >
  0x00, 0x18, 0x00, 0x18, 0x0C, 0x6C, 0x18, 0x00, // ?
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // @
  0x00, 0x66, 0x66, 0x7E, 0x66, 0x66, 0x3C, 0x00, // A
  0x00, 0x7C, 0x66, 0x66, 0x7C, 0x66, 0x7C, 0x00, // B
  0x00, 0x3C, 0x66, 0x60, 0x60, 0x66, 0x3C, 0x00, // C
  0x00, 0x7C, 0x66, 0x66, 0x66, 0x66, 0x7C, 0x00, // D
  0x00, 0x7E, 0x60, 0x60, 0x7C, 0x60, 0x7E, 0x00, // E
  0x00, 0x60, 0x60, 0x60, 0x7C, 0x60, 0x7E, 0x00, // F
  0x00, 0x3E, 0x66, 0x66, 0x6E, 0x60, 0x3E, 0x00, // G
  0x00, 0x66, 0x66, 0x66, 0x7E, 0x66, 0x66, 0x00, // H
  0x00, 0x3C, 0x18, 0x18, 0x18, 0x18, 0x3C, 0x00, // I
  0x00, 0x18, 0x58, 0x18, 0x18, 0x18, 0x3C, 0x00, // J
  0x00, 0x6C, 0x6C, 0x78, 0x78, 0x6C, 0x66, 0x00, // K
  0x00, 0x7E, 0x60, 0x60, 0x60, 0x60, 0x60, 0x00, // L
  0x00, 0x66, 0x66, 0x7E, 0x7E, 0x66, 0x42, 0x00, // M
  0x00, 0x6E, 0x6E, 0x7E, 0x7E, 0x76, 0x66, 0x00, // N
  0x00, 0x3C, 0x66, 0x66, 0x66, 0x66, 0x3C, 0x00, // O
  0x00, 0x60, 0x60, 0x7C, 0x66, 0x66, 0x7C, 0x00, // P
  0x00, 0x2E, 0x68, 0x66, 0x66, 0x66, 0x3C, 0x00, // Q
  0x00, 0x66, 0x66, 0x7C, 0x66, 0x66, 0x7C, 0x00, // R
  0x00, 0x7C, 0x06, 0x3E, 0x7C, 0x60, 0x3E, 0x00, // S
  0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x7E, 0x00, // T
  0x00, 0x3C, 0x66, 0x66, 0x66, 0x66, 0x66, 0x00, // U
  0x00, 0x18, 0x24, 0x66, 0x66, 0x66, 0x66, 0x00, // V
  0x00, 0x24, 0x7E, 0x7E, 0x66, 0x66, 0x66, 0x00, // W
  0x00, 0x66, 0x34, 0x18, 0x18, 0x2C, 0x66, 0x00, // X
  0x00, 0x18, 0x18, 0x18, 0x3C, 0x66, 0x66, 0x00, // Y
  0x00, 0x7E, 0x70, 0x38, 0x1C, 0x0E, 0x7E, 0x00, // Z
  0x00, 0x0C, 0x08, 0x08, 0x08, 0x08, 0x0C, 0x00, // [
  0x00, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x00,
  0x00, 0x30, 0x10, 0x10, 0x10, 0x10, 0x30, 0x00, // ]
  0x00, 0x00, 0x00, 0x00, 0x00, 0x24, 0x18, 0x00, // ^
  0x00, 0x7E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // _
  0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x20, 0x00, // `
  0x00, 0x66, 0x66, 0x7E, 0x66, 0x66, 0x3C, 0x00, // a
  0x00, 0x7C, 0x66, 0x66, 0x7C, 0x66, 0x7C, 0x00, // b
  0x00, 0x3C, 0x66, 0x60, 0x60, 0x66, 0x3C, 0x00, // c
  0x00, 0x7C, 0x66, 0x66, 0x66, 0x66, 0x7C, 0x00, // d
  0x00, 0x7E, 0x60, 0x60, 0x7C, 0x60, 0x7E, 0x00, // e
  0x00, 0x60, 0x60, 0x60, 0x7C, 0x60, 0x7E, 0x00, // f
  0x00, 0x3E, 0x66, 0x66, 0x6E, 0x60, 0x3E, 0x00, // g
  0x00, 0x66, 0x66, 0x66, 0x7E, 0x66, 0x66, 0x00, // h
  0x00, 0x3C, 0x18, 0x18, 0x18, 0x18, 0x3C, 0x00, // i
  0x00, 0x18, 0x58, 0x18, 0x18, 0x18, 0x3C, 0x00, // j
  0x00, 0x6C, 0x6C, 0x78, 0x78, 0x6C, 0x66, 0x00, // k
  0x00, 0x7E, 0x60, 0x60, 0x60, 0x60, 0x60, 0x00, // l
  0x00, 0x66, 0x66, 0x7E, 0x7E, 0x66, 0x42, 0x00, // m
  0x00, 0x6E, 0x6E, 0x7E, 0x7E, 0x76, 0x66, 0x00, // n
  0x00, 0x3C, 0x66, 0x66, 0x66, 0x66, 0x3C, 0x00, // o
  0x00, 0x60, 0x60, 0x7C, 0x66, 0x66, 0x7C, 0x00, // p
  0x00, 0x2E, 0x68, 0x66, 0x66, 0x66, 0x3C, 0x00, // q
  0x00, 0x66, 0x66, 0x7C, 0x66, 0x66, 0x7C, 0x00, // r
  0x00, 0x7C, 0x06, 0x3E, 0x7C, 0x60, 0x3E, 0x00, // s
  0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x7E, 0x00, // t
  0x00, 0x3C, 0x66, 0x66, 0x66, 0x66, 0x66, 0x00, // u
  0x00, 0x18, 0x24, 0x66, 0x66, 0x66, 0x66, 0x00, // v
  0x00, 0x24, 0x7E, 0x7E, 0x66, 0x66, 0x66, 0x00, // w
  0x00, 0x66, 0x34, 0x18, 0x18, 0x2C, 0x66, 0x00, // x
  0x00, 0x18, 0x18, 0x18, 0x3C, 0x66, 0x66, 0x00, // y
  0x00, 0x7E, 0x70, 0x38, 0x1C, 0x0E, 0x7E, 0x00, // z
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // |
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // }
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // ~
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
} };

unsigned long Font2_Color    = 0xFFFFFF;
unsigned long Font2_DefColor = 0xFFFFFF;

unsigned char ALIGN(4) Font2_Data[1024] = {
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x0C, 0x00, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x24, 0x12, 0x36,
  0x00, 0x36, 0x7E, 0x37, 0x36, 0x76, 0x3F, 0x36,
  0x00, 0x14, 0x7E, 0x15, 0x3E, 0x54, 0x3F, 0x14,
  0x00, 0x27, 0x15, 0x17, 0x08, 0x74, 0x54, 0x72,
  0x00, 0x3B, 0x64, 0x66, 0x2A, 0x19, 0x2C, 0x18,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x10, 0x30,
  0x00, 0x0C, 0x18, 0x18, 0x18, 0x18, 0x18, 0x0C,
  0x00, 0x18, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x18,
  0x00, 0x00, 0x33, 0x1E, 0x0C, 0x1E, 0x33, 0x00,
  0x00, 0x00, 0x0C, 0x0C, 0x3F, 0x0C, 0x0C, 0x00,
  0x00, 0x10, 0x08, 0x18, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x3E, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x20, 0x10, 0x10, 0x08, 0x04, 0x04, 0x02,
  0x00, 0x1C, 0x22, 0x63, 0x63, 0x63, 0x22, 0x1C,
  0x00, 0x0C, 0x0C, 0x0C, 0x0C, 0x1C, 0x0C, 0x0C,
  0x00, 0x7F, 0x70, 0x38, 0x0E, 0x63, 0x63, 0x3E,
  0x00, 0x3E, 0x63, 0x03, 0x1E, 0x03, 0x63, 0x3E,
  0x00, 0x06, 0x7F, 0x66, 0x66, 0x36, 0x1E, 0x0E,
  0x00, 0x3E, 0x63, 0x03, 0x7E, 0x60, 0x60, 0x7F,
  0x00, 0x3E, 0x63, 0x63, 0x7E, 0x60, 0x63, 0x3E,
  0x00, 0x0C, 0x0C, 0x0C, 0x06, 0x63, 0x63, 0x7F,
  0x00, 0x3E, 0x63, 0x63, 0x3E, 0x63, 0x63, 0x3E,
  0x00, 0x3E, 0x63, 0x03, 0x3F, 0x63, 0x63, 0x3E,
  0x00, 0x00, 0x0C, 0x00, 0x00, 0x00, 0x0C, 0x00,
  0x00, 0x08, 0x04, 0x0C, 0x00, 0x00, 0x0C, 0x00,
  0x00, 0x00, 0x06, 0x0C, 0x18, 0x18, 0x0C, 0x06,
  0x00, 0x00, 0x00, 0x3E, 0x00, 0x00, 0x3E, 0x00,
  0x00, 0x00, 0x18, 0x0C, 0x06, 0x06, 0x0C, 0x18,
  0x00, 0x0C, 0x00, 0x0C, 0x06, 0x33, 0x33, 0x1E,
  0x3C, 0x42, 0x99, 0xA5, 0xA1, 0x99, 0x42, 0x3C,
  0x00, 0x63, 0x63, 0x7F, 0x63, 0x63, 0x63, 0x3E,
  0x00, 0x7E, 0x63, 0x63, 0x7E, 0x63, 0x63, 0x7E,
  0x00, 0x3E, 0x63, 0x60, 0x60, 0x60, 0x63, 0x3E,
  0x00, 0x7C, 0x66, 0x63, 0x63, 0x63, 0x66, 0x7C,
  0x00, 0x7F, 0x60, 0x60, 0x7E, 0x60, 0x60, 0x7F,
  0x00, 0x60, 0x60, 0x60, 0x7E, 0x60, 0x60, 0x7F,
  0x00, 0x3E, 0x63, 0x63, 0x6F, 0x60, 0x63, 0x3E,
  0x00, 0x63, 0x63, 0x63, 0x7F, 0x63, 0x63, 0x63,
  0x00, 0x1E, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x1E,
  0x00, 0x1C, 0x26, 0x06, 0x06, 0x06, 0x06, 0x3F,
  0x00, 0x63, 0x6E, 0x78, 0x70, 0x78, 0x6E, 0x63,
  0x00, 0x7F, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60,
  0x00, 0x63, 0x63, 0x6B, 0x6B, 0x77, 0x77, 0x63,
  0x00, 0x63, 0x63, 0x67, 0x6F, 0x7B, 0x73, 0x63,
  0x00, 0x3E, 0x63, 0x63, 0x63, 0x63, 0x63, 0x3E,
  0x00, 0x60, 0x60, 0x7E, 0x63, 0x63, 0x63, 0x7E,
  0x00, 0x3D, 0x66, 0x6B, 0x63, 0x63, 0x63, 0x3E,
  0x00, 0x63, 0x66, 0x6C, 0x7E, 0x63, 0x63, 0x7E,
  0x00, 0x3E, 0x63, 0x03, 0x3E, 0x60, 0x63, 0x3E,
  0x00, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x3F,
  0x00, 0x3E, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63,
  0x00, 0x1C, 0x1C, 0x36, 0x36, 0x77, 0x63, 0x63,
  0x00, 0x36, 0x3E, 0x6B, 0x6B, 0x6B, 0x6B, 0x63,
  0x00, 0x63, 0x77, 0x3E, 0x1C, 0x3E, 0x77, 0x63,
  0x00, 0x0C, 0x0C, 0x0C, 0x1E, 0x1E, 0x33, 0x33,
  0x00, 0x7F, 0x60, 0x30, 0x1C, 0x06, 0x03, 0x7F,
  0x00, 0x3C, 0x30, 0x30, 0x30, 0x30, 0x30, 0x30,
  0x3C, 0x02, 0x04, 0x04, 0x08, 0x10, 0x10, 0x20,
  0x3C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x3C,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x66, 0x7E, 0x3C,
  0x00, 0x7E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x38, 0x30,
  0x00, 0x3B, 0x67, 0x63, 0x3F, 0x03, 0x3E, 0x00,
  0x00, 0x7E, 0x63, 0x63, 0x63, 0x7E, 0x60, 0x60,
  0x00, 0x3E, 0x63, 0x60, 0x63, 0x3E, 0x00, 0x00,
  0x00, 0x3F, 0x63, 0x63, 0x63, 0x3F, 0x03, 0x03,
  0x00, 0x3F, 0x60, 0x7E, 0x63, 0x3E, 0x00, 0x00,
  0x18, 0x0C, 0x0C, 0x0C, 0x0C, 0x3F, 0x0C, 0x06,
  0x3E, 0x63, 0x3E, 0x04, 0x1E, 0x33, 0x1E, 0x04,
  0x00, 0x63, 0x63, 0x63, 0x7E, 0x60, 0x60, 0x60,
  0x00, 0x1E, 0x0C, 0x0C, 0x0C, 0x1E, 0x00, 0x0C,
  0x1C, 0x26, 0x06, 0x06, 0x06, 0x0F, 0x00, 0x06,
  0x00, 0x33, 0x36, 0x3C, 0x36, 0x33, 0x30, 0x30,
  0x00, 0x1E, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x1C,
  0x00, 0x6B, 0x6B, 0x6B, 0x7F, 0x6A, 0x00, 0x00,
  0x00, 0x63, 0x63, 0x73, 0x7F, 0x6E, 0x00, 0x00,
  0x00, 0x3E, 0x63, 0x63, 0x63, 0x3E, 0x00, 0x00,
  0x60, 0x7C, 0x63, 0x63, 0x63, 0x7C, 0x00, 0x00,
  0x03, 0x3F, 0x63, 0x63, 0x63, 0x3F, 0x00, 0x00,
  0x00, 0x30, 0x30, 0x38, 0x3E, 0x36, 0x00, 0x00,
  0x00, 0x7E, 0x03, 0x3E, 0x60, 0x3F, 0x00, 0x00,
  0x00, 0x07, 0x0C, 0x0C, 0x0C, 0x3F, 0x0C, 0x0C,
  0x00, 0x3B, 0x67, 0x63, 0x63, 0x63, 0x00, 0x00,
  0x00, 0x1C, 0x3E, 0x36, 0x63, 0x63, 0x00, 0x00,
  0x00, 0x36, 0x7F, 0x6B, 0x6B, 0x6B, 0x00, 0x00,
  0x00, 0x63, 0x77, 0x3E, 0x77, 0x63, 0x00, 0x00,
  0x1E, 0x03, 0x1F, 0x33, 0x33, 0x33, 0x00, 0x00,
  0x00, 0x7F, 0x30, 0x1C, 0x06, 0x7F, 0x00, 0x00,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0x00, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF,
  0xFF, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xFF
};

unsigned long Font3_Color    = 0xFFFFFF;
unsigned long Font3_DefColor = 0xFFFFFF;

unsigned char ALIGN(4) Font3_Data[512] = {
  0xF0, 0x99, 0x99, 0x0F,
  0xF0, 0x99, 0x99, 0x0F,
  0xF0, 0x99, 0x99, 0x0F,
  0xF0, 0x99, 0x99, 0x0F,
  0xF0, 0x99, 0x99, 0x0F,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00,
  0x40, 0x40, 0x44, 0x00, // !
  0x00, 0x00, 0xAA, 0x00, // "
  0xA0, 0xAE, 0xAE, 0x00, // #
  0xC0, 0x46, 0x6C, 0x00, // $
  0x20, 0x48, 0x82, 0x00, // %
  0x60, 0x4A, 0x4A, 0x00, // &
  0x00, 0x00, 0x44, 0x00, // '
  0x40, 0x88, 0x48, 0x00, // (
  0x40, 0x22, 0x42, 0x00, // )
  0x00, 0x4A, 0x0A, 0x00, // *
  0x00, 0xE4, 0x04, 0x00, // +
  0x48, 0x04, 0x00, 0x00, // ,
  0x00, 0xE0, 0x00, 0x00, // -
  0x40, 0x00, 0x00, 0x00, // .
  0x80, 0x48, 0x24, 0x00,
  0x40, 0xAA, 0x4A, 0x00, // 0
  0x40, 0x44, 0x4C, 0x00, // 1
  0xE0, 0x48, 0xC2, 0x00, // 2
  0xC0, 0xC2, 0xC2, 0x00, // 3
  0x20, 0xAE, 0x48, 0x00, // 4
  0xC0, 0xC2, 0xE8, 0x00, // 5
  0x40, 0xCA, 0x48, 0x00, // 6
  0x40, 0x44, 0xE2, 0x00, // 7
  0x40, 0x4A, 0x4A, 0x00, // 8
  0x40, 0x62, 0x4A, 0x00, // 9
  0x40, 0x00, 0x04, 0x00, // :
  0x48, 0x40, 0x00, 0x00, //, //
  0x20, 0x84, 0x24, 0x00, // <
  0x00, 0x0E, 0x0E, 0x00, // =
  0x80, 0x24, 0x84, 0x00, // >
  0x04, 0x24, 0x4A, 0x00, // ?
  0x00, 0xA6, 0x46, 0x00, // @
  0xA0, 0xEA, 0x4A, 0x00, // A
  0xC0, 0xCA, 0xCA, 0x00, // B
  0x60, 0x88, 0x68, 0x00, // C
  0xC0, 0xAA, 0xCA, 0x00, // D
  0xE0, 0xC8, 0xE8, 0x00, // E
  0x80, 0xC8, 0xE8, 0x00, // F
  0x60, 0xAA, 0x68, 0x00, // G
  0xA0, 0xEA, 0xAA, 0x00, // H
  0x40, 0x44, 0x44, 0x00, // I
  0x40, 0x2A, 0x22, 0x00, // J
  0xA0, 0xCA, 0xAA, 0x00, // K
  0xE0, 0x88, 0x88, 0x00, // L
  0xA0, 0xAA, 0xAE, 0x00, // M
  0xA0, 0xAA, 0xCA, 0x00, // N
  0xE0, 0xAA, 0xEA, 0x00, // O
  0x80, 0xC8, 0xCA, 0x00, // P
  0xE0, 0xAB, 0xEA, 0x00, // Q
  0xA0, 0xCA, 0xCA, 0x00, // R
  0xC0, 0x42, 0x68, 0x00, // S
  0x40, 0x44, 0xE4, 0x00, // T
  0x40, 0xAA, 0xAA, 0x00, // U
  0x40, 0xA4, 0xAA, 0x00, // V
  0xA0, 0xAE, 0xAA, 0x00, // W
  0xA0, 0x4A, 0xAA, 0x00, // X
  0x40, 0x44, 0xAA, 0x00, // Y
  0xE0, 0x48, 0xE2, 0x00, // Z
  0x60, 0x44, 0x64, 0x00, // [
  0x20, 0x42, 0x84, 0x00,
  0xC0, 0x44, 0xC4, 0x00, // ]
  0x00, 0xA0, 0x04, 0x00, // ^
  0x0E, 0x00, 0x00, 0x00, // _
  0x00, 0x40, 0x08, 0x00, // `
  0x60, 0xAA, 0x06, 0x00, // a
  0xC0, 0xCA, 0x88, 0x00, // b
  0x60, 0x88, 0x06, 0x00, // c
  0x60, 0x6A, 0x22, 0x00, // d
  0x60, 0xAC, 0x06, 0x00, // e
  0x80, 0xC8, 0x68, 0x00, // f
  0x2C, 0xA6, 0x06, 0x00, // g
  0xA0, 0xCA, 0x88, 0x00, // h
  0x40, 0x44, 0x40, 0x00, // i
  0x48, 0x44, 0x40, 0x00, // j
  0xA0, 0xAC, 0x0A, 0x00, // k
  0x20, 0x44, 0x44, 0x00, // l
  0xA0, 0xEA, 0x0A, 0x00, // m
  0xA0, 0xAA, 0x0C, 0x00, // n
  0x40, 0xAA, 0x04, 0x00, // o
  0x88, 0xAC, 0x04, 0x00, // p
  0x22, 0xA6, 0x04, 0x00, // q
  0x80, 0xC8, 0x0A, 0x00, // r
  0xC0, 0x86, 0x06, 0x00, // s
  0x20, 0x44, 0x46, 0x00, // t
  0x60, 0xAA, 0x0A, 0x00, // u
  0x40, 0xAA, 0x0A, 0x00, // v
  0xA0, 0xAE, 0x0A, 0x00, // w
  0xA0, 0x4A, 0x0A, 0x00, // x
  0x40, 0xA4, 0x0A, 0x00, // y
  0xE0, 0x48, 0x0E, 0x00, // z
  0x60, 0xC4, 0x64, 0x00, // {
  0x40, 0x04, 0x44, 0x00, // |
  0xC0, 0x64, 0xC4, 0x00, // }
  0x00, 0x00, 0xC2, 0x00, // ~
  0x00, 0xAE, 0x04, 0x00
};
